import pygame, copy, random
from euclid import *
from pygame.locals import *


BGCOLOR=[50,100,200]
################   Function    ###########################

def rect(c,r):
    """Takes a center and offset and returns a list of points for that rectangle."""
    return [Point2(c[0]+r[0],c[1]+r[1]),
            Point2(c[0]+r[0],c[1]-r[1]),
            Point2(c[0]-r[0],c[1]-r[1]),
            Point2(c[0]-r[0],c[1]+r[1])]



def triangle(c,bh):
    """Take a center, anf a base-height tuple and makes the poins for a triangle)"""
    return [Point2(c[0]-bh[0]/2,c[1]-bh[1]/2),
            Point2(c[0]        ,c[1]+bh[1]  ),
            Point2(c[0]+bh[0]/2,c[1]-bh[1]/2)]

def lerp(a,b,p=0.7):
    """Blend two colors with a certain percentage of the old"""
    newcolor=[0,0,0]
    for i in range(3):
        newcolor[i]=int(a[i]*p+b[i]*(1-p))
    return newcolor
        
def fcenter(points):
    """ Takes the average of x and y to find the center point or something close """
    return Point2(sum([p[0]/len(points) for p in points]),
                  sum([p[1]/len(points) for p in points]))

def collision(player,wall,truefalse=True):
    """Detects if there is a collison. Optionally returns a vector"""
    vlist=[]
    if truefalse:
        return player.rect.colliderect(wall)
    else:
        if player.rect.colliderect(wall):
            
            if player.vel.x>0 and player.rect.top+2>wall.rect.top and player.oldrect.left < wall.rect.left: vlist.append(Vector2(wall.rect.left-player.rect.right,0))
            if player.vel.x<0 and player.rect.top+2>wall.rect.top and player.oldrect.right > wall.rect.right: vlist.append(Vector2(wall.rect.right-player.rect.left,0))
            if player.vel.y>0 and player.oldrect.top<wall.rect.top: vlist.append(Vector2(0,wall.rect.top-player.rect.bottom))
                             
            if len(vlist)>0:
                return vlist[0]
            
            else: return Vector2(0,0)
            
        else:
            return Vector2(0,0)



################    Classes    #########################

class polygon(object):
    """Basic polygon object"""
    def __init__(self,points,color=[200,200,100]):
        self.color=color
        self.points=[Point2(*x) for x in points]
        self.center=fcenter(points)
        self.vel=Vector2(0,0)
        self.acc=Vector2(0,0)
        self.width=0
        self.rect=pygame.Rect(20,20,20,20)
        self.oldrect=copy.copy(self.rect)
        self.oldpoints=self.points[:]

        ##person stuff i'm to lazy to make into a new init.
        self.upped=False
        

        
    def draw(self,screen):
        """Draws a polygon based on the points and an outline that blends in with the background"""
        self.rect=pygame.draw.polygon(
            screen,
            self.color,
            [x[:] for x in self.points],
            self.width)

        
        pygame.draw.lines(
            screen,
            lerp(self.color,BGCOLOR),
            1,
            [x[:] for x in self.points],
            1)

    def update(self):
        """adds accell to velocity and moves based on the velocity"""
        self.vel+=self.acc
        self.moverel(self.vel)

    def moverel(self,v):
        """Takes a vector and moves the outer points and the fcenter based on those"""
        for i in self.points+[self.center]:
            i.x+=v.x
            i.y+=v.y

    def old(self):
        self.oldpoints=self.points[:]
        self.oldrect=copy.copy(self.rect)
        

class person(polygon):
    def adjust(self,walls):
        for wall in walls:
            self.vel+=0.4*collision(self,wall,False)

class log(polygon):
    def update(self,player):
        
        self.vel+=self.acc
        self.moverel(self.vel)

        if collision(player,self):
            player.upped=False
            player.moverel(self.vel)
            


        def draw(self,screen):
            self.rect=pygame.draw.polygon(
                screen,
                self.color,
                [x[:] for x in self.points],
                self.width)

        
            pygame.draw.lines(
                screen,
                lerp(self.color,BGCOLOR),
                1,
                [x[:] for x in self.points],
                1)




################ Main function ######################
def main():
    #Initialize
    pygame.init()
    screen = pygame.display.set_mode((640, 480))
    screen.fill(BGCOLOR)
    pygame.display.update()
    clock = pygame.time.Clock()
    end = False

    ## Makes the players and logs
    player=person(rect([430,320],[15,15]))
    loglist=[log(rect([500+i+random.randint(-5,5),450],[100+random.randint(-15,15),30]),[150,75,0]) for i in range(0,300*10,300)]
    for logy in loglist:
        logy.vel.x+=-2

    while not end:

        clock.tick(60)
        screen.fill(BGCOLOR)

        for l in loglist:
            l.old()
            l.update(player)
            l.draw(screen)

        player.old()
        player.update()
        player.adjust(loglist)
        player.draw(screen)

        player.vel.x*=0.9
        player.vel.y*=0.9
        player.vel.y+=0.4

        if player.center.y>500:
            end=True
        
        ## Player input
        for e in pygame.event.get():
            if e.type == QUIT:
                end = True
            if e.type == KEYDOWN:
                if e.key==K_UP and player.upped==False:
                    player.vel.y+=-12
                    player.upped=True
                if e.key==K_DOWN:
                    player.acc.y=0.7
                if e.key==K_LEFT:
                    player.acc.x=-0.7
                if e.key==K_RIGHT:
                    player.acc.x=0.7
                if e.key==K_ESCAPE:
                    end = True

            if e.type == KEYUP:
                if e.key==K_UP:
                    player.acc.y=0
                if e.key==K_DOWN and player.acc.y>0:
                    player.acc.y=0
                if e.key==K_LEFT and player.acc.x<0:
                    player.acc.x=0
                if e.key==K_RIGHT and player.acc.x>0:
                    player.acc.x=0


        pygame.display.update()

    pygame.quit()






if __name__ == "__main__": main()

